/***********************************************************
 * $Id$
 *
 * FancyLibs libmodbus wrapper
 *
 * Copyright (C) 2014 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYMODBUSCONFIG_H_
#define FANCYMODBUSCONFIG_H_

#define FANCYMODBUSCONFIG_BEAN "Fancy::Modbus::ModbusConfig"

#include "fancymodbus.h"

#include <fancyobject.h>

#include <string>


namespace Fancy {
class ConfigOptions;
namespace Modbus {

/**
 * A wrapper around a mdbus_mapping_t structure for implementing
 * sample modbus servers.
 */
class FANCYMODBUS_API ModbusConfig : virtual public Fancy::Object {

public:
	typedef Fancy::ObjPtr<ModbusConfig> ptr_t;

	/**
	 * parseForSimulatorTriggerScripts triggers parsing of on-outputbit-X-B and the like
	 */
	static ptr_t newInstance(
			const Fancy::ObjPtr<Fancy::ConfigOptions> & config,
			bool parseSimulatorOptions);

	virtual int getGlobalVerbosity() const = 0;

	virtual int getNrOutputBits() const = 0;
	virtual int getNrInputBits() const = 0;
	virtual int getNrOutputRegisters() const = 0;
	virtual int getNrInputRegisters() const = 0;

	virtual std::string getListenAddress() const = 0;

	virtual bool hasBitOutputTriggerScripts() const = 0;
	virtual std::string getBitOutputTriggerScript(int bit, bool bitSet) const = 0;


protected:
	ModbusConfig();
	virtual ~ModbusConfig();
};

} /* namespace Modbus */
} /* namespace Fancy */

#endif /* FANCYMODBUSCONFIG_H_ */
