/***********************************************************
 * $Id$
 *
 * FancyLibs libmodbus wrapper
 *
 * Copyright (C) 2014 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYMODBUSCONNECTION_H_
#define FANCYMODBUSCONNECTION_H_

#include <fancymodbus.h>
#include <fancyobject.h>

namespace Fancy {
namespace Sock {
class Address;
}

namespace Modbus {

class FANCYMODBUS_API ModbusConnection : virtual public Fancy::Object {

public:
	typedef Fancy::ObjPtr<ModbusConnection> ptr_t;

	/**
	 * Instantiate a new client instance, which will be connected when the first
	 * request is issued.
	 *
	 * If @c reconnectOnError is @c true, request are retried, when a transport error occurred.
	 */
	static ptr_t newInstance(const Fancy::ObjPtr<Fancy::Sock::Address> &addr, bool reconnectOnError);

	/**
	 * @see modbus_set_response_timeout(2)
	 */
	virtual void set_response_timeout(int millis) = 0;
	/**
	 * @see modbus_set_byte_timeout(2)
	 */
	virtual void set_byte_timeout(int millis) = 0;
	/**
	 * @see modbus_read_bits(2)
	 */
	virtual int read_bits(int addr, int nb, uint8_t *dest) = 0;
	/**
	 * @see modbus_read_input_bits(2)
	 */
	virtual int read_input_bits(int addr, int nb, uint8_t *dest) = 0;
	/**
	 * @see modbus_read_registers(2)
	 */
	virtual int read_registers(int addr, int nb, uint16_t *dest) = 0;
	/**
	 * @see modbus_read_input_registers(2)
	 */
	virtual int read_input_registers(int addr, int nb, uint16_t *dest) = 0;
	/**
	 * @see modbus_write_bit(2)
	 */
	virtual int write_bit(int coil_addr, int status) = 0;
	/**
	 * @see modbus_write_register(2)
	 */
	virtual int write_register(int reg_addr, int value) = 0;
	/**
	 * @see modbus_write_bits(2)
	 */
	virtual int write_bits(int addr, int nb, const uint8_t *data) = 0;
	/**
	 * @see modbus_write_registers(2)
	 */
	virtual int write_registers(int addr, int nb, const uint16_t *data) = 0;
	/**
	 * @see modbus_write_and_read_registers(2)
	 */
	virtual int write_and_read_registers(int write_addr, int write_nb,
	                                    const uint16_t *src, int read_addr, int read_nb,
	                                    uint16_t *dest) = 0;

	/**
	 * Forcibly disconnect this instance if it has been connected by issuing a request.
	 */
	virtual void disconnect() = 0;

	/**
	 * Set the slave ID, if you need to connect to a no-default serial
	 * device on the remote end.
	 * @see modbus_set_slave(29
	 */
	virtual void set_slave(int id) = 0;

protected:
	ModbusConnection();
	virtual ~ModbusConnection();
};

} /* namespace Modbus */
} /* namespace Fancy */

#endif /* FANCYMODBUSCONNECTION_H_ */
