/***********************************************************
 * $Id$
 *
 * FancyLibs libmodbus wrapper
 *
 * Copyright (C) 2014 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYMODBUSCONNECTIONHANDLERFACTORY_H_
#define FANCYMODBUSCONNECTIONHANDLERFACTORY_H_

#include <fancymodbus.h>
#include <fancysockconnectionhandler.h>

namespace Fancy {
namespace Modbus {

class ModbusMapping;

class FANCYMODBUS_API ModbusConnectionHandlerFactory : virtual public Fancy::Sock::ConnectionHandlerFactory {

	Fancy::ObjPtr<ModbusMapping> mapping;
public:
	typedef Fancy::ObjPtr<ModbusConnectionHandlerFactory> ptr_t;

	/**
	 * Instantiate a connection handler factory instance, which will operate on the given mapping instance.
	 */
	static ptr_t newInstance(const Fancy::ObjPtr<ModbusMapping> &mapping);

	virtual Fancy::ObjPtr<Fancy::Runnable> newConnectionHandler(const Fancy::ObjPtr<Fancy::Sock::Device> &device);

	const Fancy::ObjPtr<ModbusMapping> &getMapping() const {
		return this->mapping;
	};

protected:
	ModbusConnectionHandlerFactory(const Fancy::ObjPtr<ModbusMapping> &mapping);
	virtual ~ModbusConnectionHandlerFactory();
};

} /* namespace Modbus */
} /* namespace Fancy */

#endif /* FANCYMODBUSCONNECTIONHANDLERFACTORY_H_ */
