/***********************************************************
 * $Id$
 *
 * FancyLibs libmodbus wrapper
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYMODBUSEXCEPTION_H_
#define FANCYMODBUSEXCEPTION_H_

#include <fancyexception.h>
#include <fancymodbuslog.h>

namespace Fancy {
namespace Modbus {

class FANCYMODBUS_API ModbusException : public Fancy::Exception
{
  int err;

public:
    ModbusException (int facility,
            int priority,
            const char *fmt, ...) throw()
#ifdef __GNUG__
            __attribute__((format(printf,4,5)))
#endif
;

    ModbusException (int facility,
            int priority,
			int _err,
            const char *fmt, ...) throw()
#ifdef __GNUG__
            __attribute__((format(printf,5,6)))
#endif
;

    virtual ~ModbusException() throw();

	int getErrno() const throw();
	const char *getStrError() const throw();

    /**
     * Return whether this is a protocol exception received from the
     * server. If this is true, the protocol exception code may be
     * retrieved by #getExceptionCode()
     *
     * If this is false, the underlying TCP/IP transport layer
     * worked OK, but the received data package was inconsistent.
     *
     * Raw transport errors are reported as a Fancy::CException
     */
	bool isProtocolException() const throw();

    /**
     * Returns teh server side exception code, which start with 1,
     * if #isProtocolException() returns true.
     *
	 * The following exception code values are defined inside the
     * modbus.h header of the upstream libmodbus library:
     *
     * @see MODBUS_EXCEPTION_ILLEGAL_FUNCTION
     * @see MODBUS_EXCEPTION_ILLEGAL_DATA_ADDRESS
     * @see MODBUS_EXCEPTION_ILLEGAL_DATA_VALUE
     * @see MODBUS_EXCEPTION_SLAVE_OR_SERVER_FAILURE
     * @see MODBUS_EXCEPTION_ACKNOWLEDGE
     * @see MODBUS_EXCEPTION_SLAVE_OR_SERVER_BUSY
     * @see MODBUS_EXCEPTION_NEGATIVE_ACKNOWLEDGE
     * @see MODBUS_EXCEPTION_MEMORY_PARITY
     * @see MODBUS_EXCEPTION_NOT_DEFINED
     * @see MODBUS_EXCEPTION_GATEWAY_PATH
     * @see MODBUS_EXCEPTION_GATEWAY_TARGET
     */
    int getExceptionCode() const throw();

	void appendModbusMessageV(const char *msg, va_list ap) throw ();

#ifdef __GNUG__
	__attribute__((format(printf,2,3)))
#endif
    void appendModbusMessage(const char *msg, ...) throw ();
};

}}// end namespaces


#endif
