/***********************************************************
 * $Id$
 *
 * FancyLibs libmodbus wrapper
 *
 * Copyright (C) 2014 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYMODBUSMAPPING_H_
#define FANCYMODBUSMAPPING_H_

#include <fancymodbus.h>

#include <fancyobject.h>

#include <fancysockstateful.h>


namespace Fancy {
namespace Modbus {

/**
 * A wrapper around a mdbus_mapping_t structure for implementing
 * sample modbus servers.
 */
class FANCYMODBUS_API ModbusMapping : virtual public Fancy::Object, virtual public Fancy::Sock::Stateful {

public:
	typedef Fancy::ObjPtr<ModbusMapping> ptr_t;

	/**
	 * Instantiate a new mapping with the given maximal number of
	 * input/output bits/registers.
	 *
	 * @param nb_bits Maximal number of output bits, this is the maximal
	 *                used address of an output bit plus one.
	 * @param nb_input_bits Maximal number of input bits, this is the maximal
	 *                used address of an input bit plus one.
	 * @param nb_registers Maximal number of output registers, this is the maximal
	 *                used address of an output register plus one.
	 * @param nb_input_registers Maximal number of input registers, this is the maximal
	 *                used address of an input register plus one.
	 */
	static ptr_t newInstance(int nb_bits, int nb_input_bits, int nb_registers, int nb_input_registers);

	/**
	 * Return the pointer to @c modbus_mapping_t for using this structure
	 * in a call to @c modbus_reply.
	 */
	virtual void* getHandle() const = 0;

	virtual int getNbBits() const = 0;
	virtual int getNbInputBits() const = 0;
	virtual int getNbRegisters() const = 0;
	virtual int getNbInputRegisters() const = 0;

	virtual uint8_t *getTabBits() const = 0;
	virtual uint8_t *getTabInputBits() const = 0;
	virtual uint16_t *getTabInputRegisters() const = 0;
	virtual uint16_t *getTabRegisters() const = 0;

	// Sock::Stateful
	virtual void printStatus(std::ostream &os) const = 0;

protected:
	ModbusMapping();
	virtual ~ModbusMapping();
};

} /* namespace Modbus */
} /* namespace Fancy */

#endif /* FANCYMODBUSMAPPING_H_ */
